## LLEU project, faq product
## Copyright (C) 2000  J. David Ibez, Andres Marzal
##                     Universitat Jaume I, Spain
## email: al028823@alumail.uji.es

## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

__doc__ = """FAQ"""
__version__ = "0.1"

from Globals import HTMLFile
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem

from Shared.xmlio import XMLio
from OrderedList import OrderedList


manage_addFAQForm = HTMLFile('addFAQ', globals())
def manage_addFAQ(self, id, title='', REQUEST=None):
  """ """
  self._setObject(id, FAQ(id, title))
  if REQUEST is not None:
    return self.manage_main(self, REQUEST)


class Entry(SimpleItem, XMLio):
  meta_type = 'FAQ Entry'

  manage_options = ({'label':'Edit', 'action':'editForm'},
                    {'label':'View', 'action':'index_html'},
                    {'label':'Security', 'action':'manage_access'})

  __ac_permissions__ = (('Edit FAQ', ['edit']),)

  def __init__(self, id, question, answer, REQUEST=None):
    self.id = id
    self.edit(question, answer)

  def edit(self, question, answer):
    """ """
    self.question = question
    self.answer = answer

  editForm = HTMLFile('editEntry', globals())

  ## XML support
  xml_name = 'entry'
  xml_elements = ('question', 'answer')

class Category(Folder, XMLio):
  meta_type = 'FAQ Category'

  manage_options = ({'label':'Entries', 'action':'entriesEditor'},
                    {'label':'Properties', 'action':'manage_propertiesForm'},
                    {'label':'View', 'action':'index_html'},
                    {'label':'Security', 'action':'manage_access'})

  __ac_permissions__ = Folder.__ac_permissions__ + (
    ('Edit FAQ', ['edit', 'addEntry', 'delEntries', 'rearrange',
                  'entriesEditor']),)

  def __init__(self, id, title):
    self.id = id
    self.title = title
    self.entries = OrderedList()

  ## Public API
  def addEntry(self, position, question, answer, REQUEST=None):
    """ """
    id = self.newId()
    self._setObject(id, Entry(id, question, answer))
    self.entries.insert(position, id)
    if REQUEST is not None:
      return self.entriesEditor(self, REQUEST)

  def delEntries(self, entries_to_remove, REQUEST=None):
    """ """
    self.manage_delObjects(entries_to_remove)
    for id in entries_to_remove:
      self.entries.remove(id)
    if REQUEST is not None:
      return self.entriesEditor(self, REQUEST)

  def getEntries(self):
    """ """
    return map(lambda x,s=self: getattr(s, x), self.entries)

  def rearrange(self, new_positions, REQUEST=None):
    """ """
    change = self.entries.rearrange(new_positions)
    if change and REQUEST is not None:
      return self.entriesEditor(self, REQUEST)

  entriesEditor = HTMLFile('entriesEditor', globals())
  index_html = HTMLFile('viewCategory', globals())

  ## XML support
  xml_name = 'category'
  xml_attributes = ('id', 'title')

  def xml_get_elements(self):
    elements = []
    for id in self.entries:
      entry = getattr(self, id)
      elements.append(entry.toXML())
    return elements

  def _xml_start_entry(self, attributes):
    self.addEntry(len(self.entries), '', '')
    return self[self.entries[-1]]


class FAQ(Folder, XMLio):
  meta_type = 'FAQ'

  manage_options = (
    {'label':'Categories', 'action':'categoriesEditor'},
    {'label':'Properties', 'action':'manage_propertiesForm'},
    {'label':'View', 'action':'index_html'},
    {'label':'XML', 'action':'XMLForm'},
    {'label':'Security', 'action':'manage_access'})

  __ac_permissions__ = Folder.__ac_permissions__ + (
    ('Edit FAQ', ['edit', 'addCategory', 'delCategories', 'rearrange',
                  'categoriesEditor']),)

  def __init__(self, id, title):
    """ """
    self.id = id
    self.title = title
    self.categories = OrderedList()
    self.global_counter = 0L

  def newId(self):
    self.global_counter = self.global_counter + 1
    return "faq" + str(self.global_counter - 1)

  ## Public API
  def addCategory(self, id, title, position, REQUEST=None):
    """ """
    self._setObject(id, Category(id, title))
    self.categories.insert(position, id)
    if REQUEST is not None:
      return self.categoriesEditor(self, REQUEST)

  def delCategories(self, categories_to_remove, REQUEST=None):
    """ """
    self.manage_delObjects(categories_to_remove)
    for id in categories_to_remove:
      self.categories.remove(id)
    if REQUEST is not None:
      return self.categoriesEditor(self, REQUEST)

  def getCategories(self):
    """ """
    return map(lambda x,s=self: getattr(s, x), self.categories)

  def rearrange(self, new_positions, REQUEST=None):
    """ """
    change = self.categories.rearrange(new_positions)
    if change and REQUEST is not None:
      return self.categoriesEditor(self, REQUEST)

  categoriesEditor = HTMLFile('categoriesEditor', globals())
  index_html = HTMLFile('viewFAQ', globals())

  ## XML support
  xml_name = 'faq'
  xml_attributes = ('id', 'title')

  def xml_get_elements(self):
    elements = []
    for id in self.categories:
      category = getattr(self, id)
      elements.append(category.toXML())
    return elements

  def _xml_start_category(self, attributes):
    id = attributes['id']
    title = attributes['title']
    self.addCategory(id, title, len(self.categories))
    return self[id]
